#!/usr/bin/python
from optparse import OptionParser
import math
import itertools

def toNumber(s):
    try:
        float(s)
        return float(s)	#True
    except ValueError:
        return s		#False

def pow2(f):
	return f*f

def getAvg(v):
	return math.fsum(v) / len(v)

def getStdDev(v):
	mean = getAvg(v)
	try:
		rtn = math.sqrt(math.fsum(map(pow2,v)) / len(v) - pow2(mean))
		return rtn
	except ValueError:
		return 0

# calculate confidence interval using scipy based on numpy

class StatsRUN:
	def __init__(self):
		self.title = ""
		self.data = []
		self.datacw = []
		self.out = []	

	def add(self, data):
		if self.title == "":
			self.title = data[0][:-1]	# get rid of the last '\n'
		self.data.append(map(toNumber,data[1].split()))

	def calculate(self):
		# sum column of a "matrix" using array (normal format)
		# sum(m[i][1] for i in range(10)) see http://adorio-research.org/wordpress/?p=10979
		
		# rotate cw to facilitate sum function
		# rotated_ccw = zip(*original)[::-1]
		self.datacw = zip(*self.data[::-1])
		
		self.out.append( map(getAvg, self.datacw) )
		self.out.append( map(getStdDev, self.datacw) )

	def output(self):
		
		print "item", self.title
		
		for i in range(len(self.data)):
			print i, " ".join(map(str,self.data[i]))

		print "a", " ".join(map(str,self.out[0]))
		print "v", " ".join(map(str,self.out[1]))

class StatsBUFCAP:
	def __init__(self):
		self.title = ""
		self.data = []

	def add(self, data):
		if self.title == "":
			self.title = data[0].split()[1:]
		self.data.append(data[-2].split()[1:])

	def calculate(self):
		pass

	def output(self):
		print " ".join(self.title)	
		for line in self.data:
			print " ".join(line)

class StatsWLD:
	def __init__(self,wld):
		self.title = ""
		self.wld = wld.split()
		self.data = []

	def add(self, data):
		if self.title == "":
			self.title = data[0].split()
		self.data.append(data[1].split())
		
	#def add(self, data):
	#	if self.title == "":
	#		self.title = list(itertools.chain(*zip(map(lambda x: x+'1',data[0].split()),map(lambda x: x+'0.1',data[0].split()),map(lambda x: x+'8',data[0].split()))))
	#	self.data.append(list(itertools.chain(*zip(data[1].split(),data[2].split(),data[3].split()))))
		# make 3 lines of metrics in different BUFCAP settings into 1 line (eta1, eta2, eta3, tcp1, tcp2 ....)

	def calculate(self):
		pass

	def output(self):
		print "W", " ".join(self.title)
		for i in range(len(self.data)):
			print self.wld[i], " ".join(self.data[i])


def main():
	parser = OptionParser()
	parser.add_option('-l', '--level', type='str', dest='level', default='r', help='level of statistics')
	parser.add_option('-w', '--workload', type='str', dest='wld', default='0 1 2 3 4 5 6 7 8 9', help='workload')
	opts, args = parser.parse_args()

	if opts.level == 'r':
		s = StatsRUN()
	elif opts.level == 'c':
		s = StatsBUFCAP()
	elif opts.level == 'l':
		s = StatsWLD(opts.wld)

	for arg in args:
		f = open(arg, 'r')
		fdata = f.readlines()
		f.close()
		s.add(fdata)

	s.calculate()
	s.output()

	#if len(args) < 1:
	#	print "usage: \n\n\
  	#	stat-process.py [options] {statfiles}"
	#	sys.exit()
	
if __name__ == '__main__':
	main()

